from typing import Dict
from dbgen.core.parsing import parse_line

def parse_psp_vasp(outcar   : str) -> str:
    """
    Don't know how to do this yet...
    ideally we parse the potcar to reconstruct setups dictionary
    """
    return '{}'
