from typing import Dict,Optional
from dbgen.core.parsing import parse_line

def get_dipol_qe(log:str) -> Optional[str]:
    """
    Location of dipole initial guess ... don't know how to parse (if even possible)
    """
    return None
