from typing import Dict,Optional
from dbgen.core.parsing import parse_line

def get_nmix_vasp(outcar:str) -> Optional[float]:
    """
    What vasp keyword corresponds to this, again?
    """
    return None
